@@ -1224,17 +1224,21 @@ def _fit_loglog(
         def f(logx, log_prefactor, exponent):
             return log_prefactor + exponent * logx
 
-        param, _ = curve_fit(f, logx, logy, **fit_opts)
+        param, pcov = curve_fit(f, logx, logy, **fit_opts)
         prefactor = np.exp(param[0])
         exponent = param[1]
+        perr = np.exp(np.sqrt(pcov[0, 0]))
+        eerr = np.sqrt(pcov[1, 1])
 
     elif prefactor is None:
 
         def f(logx, log_prefactor):
             return log_prefactor + exponent * logx
 
-        param, _ = curve_fit(f, logx, logy, **fit_opts)
+        param, pcov = curve_fit(f, logx, logy, **fit_opts)
         prefactor = np.exp(param[0])
+        perr = np.exp(np.sqrt(pcov[0, 0]))
+        eerr = 0
 
     elif exponent is None:
 
@@ -1243,16 +1247,19 @@ def _fit_loglog(
         def f(logx, exponent):
             return log_prefactor + exponent * logx
 
-        param, _ = curve_fit(f, logx, logy, **fit_opts)
+        param, pcov = curve_fit(f, logx, logy, **fit_opts)
         exponent = param[0]
+        perr = 0
+        eerr = np.sqrt(pcov[0, 0])
 
-    return prefactor, exponent
+    return prefactor, exponent, perr, eerr
 
 
 def fit_powerlaw(
     xdata: ArrayLike,
     ydata: ArrayLike,
     yerr: ArrayLike = None,
+    yerr_mode: str = "differentials",
     prefactor: float = None,
     exponent: float = None,
     axis: plt.Axes = None,
@@ -1260,43 +1267,67 @@ def fit_powerlaw(
     auto_fmt: str = None,
     extrapolate: bool | dict = False,
     **kwargs,
-):
+) -> (float, float, dict):
     r"""
-    Fit a powerlaw :math:`y = c x^b` by converting
-    both :math:`x` and :math:`y` to their logarithm and fitting a straight line.
-    This function does not support more customised operation like fitting an offset,
-    but custom code can be easily written by copy/pasting from here.
+    Fit a powerlaw :math:`y = c x^b` by a linear fitting of
+    :math:`\ln y = \ln c + b \ln x`.
 
-    Warning: If this function is used to plot the fit, beware that the fit is plotted using just
-    two data-points if the axis is set to log-log scale
-    (as the fit will be a straight line on that scale).
+    .. note::
+
+        This function does not support more customised operation like fitting an offset,
+        but custom code can be easily written by copy/pasting from here.
+
+    .. warning::
+
+        If this function is used to plot the fit, beware that the fit is plotted using just
+        two data-points if the axis is set to log-log scale
+        (as the fit will be a straight line on that scale).
+
+    Different modes are available to treat an error estimate (``yerr``) in ``ydata``:
+
+    *   ``"differentials"``: assume that ``yerr << ydata``, such that
+
+        .. math::
+
+            z &\equiv \ln y \\
+            \delta z &= \left| \frac{\partial z}{\partial y} \right| \delta y \\
+            \delta z &= \frac{\delta y}{y}
 
     :param xdata: Data points along the x-axis.
     :param ydata: Data points along the y-axis.
-    :param yerr: Error-bar for ``ydata``.
+    :param yerr: Error-bar for ``ydata`` (should be the standard deviation).
+    :param yerr_mode: How to treat the error in ``ydata``, see above.
     :param prefactor: Prefactor :math:`c` (fitted if not specified).
     :param exponent: Exponent :math:`b` (fitted if not specified).
     :param axis: Axis to plot along (not plotted if not specified).
-    :param fmt: Format for the label (if plotting). E.g. ``r"${0:.3f} x^{{{1:.2f}}}$"``.
+    :param fmt: Format for the label, e.g. ``r"${prefactor:.2f} x^{{{exponent:.2f}}}$"``.
     :param auto_fmt:
-        Format label (if plotting) as ``r"${0:.3f} x^{{{1:.2f}}}$"``,
+        Format label as ``r"$({prefactor:.2f} \pm {prefactor_error:.2f})
+        x^{{{exponent:.2f} \pm {exponent_error:.2f}}}$"``,
         with ``x`` replaced with the specified string.
 
     :param extrapolate:
         Plot the powerlaw on the full range of ``axis.get_xlim()``.
-        Instead of ``True'', one can specify plot options for the extrapolated line, e.g.
+        Instead of ``True``, one can specify plot options for the extrapolated line, e.g.
         ``..., extrapolate=dict(ls="--", c="r"), ...``.
 
     :param kwargs:
         Other plot options.
 
     :return:
-        ``prefactor, exponent[, plot_details]``
+        ``prefactor, exponent, details``
         The (fitted) prefector and exponent.
-        If plotting, the following details are return as dictionary::
+        The details are a dictionary as follows::
 
-            handle: Handle of the plot.
-            label: Label (if ``fmt`` was specified).
+            prefactor: (Fitted) prefactor.
+            exponent: (Fitted) exponent.
+            prefactor_error: Estimated error of prefactor.
+            exponent_error: Estimated error of exponent.
+            pcov: Covariance of fit.
+            label: Label.
+            handle: Handle of the plot (if ``axis`` was specified).
+            handle_lower: Handle of the plot of the lower extrapolation, if present.
+            handle_upper: Handle of the plot of the upper extrapolation, if present.
     """
 
     xdata = np.array(xdata)
@@ -1311,22 +1342,47 @@ def fit_powerlaw(
     logy = logy[~j]
 
     fit_opts = {}
+    details = {}
 
     if yerr is not None:
-        logyerr = np.log(np.array(yerr)[i])
-        logyerr = logyerr[~j]
-        fit_opts["sigma"] = logyerr
-        fit_opts["absolute_sigma"] = True
+        if yerr_mode.lower() == "differentials":
+            yerr = np.array(yerr)
+            sigma = yerr[i][~j] / ydata[i][~j]
+            sigma[yerr[i][~j] == 0] = np.finfo(sigma.dtype).eps  # avoid zero division
+            fit_opts["sigma"] = sigma
+            fit_opts["absolute_sigma"] = False
+        else:
+            raise OSError("yerr_mode: did you mean 'differentials'?")
 
-    prefactor, exponent = _fit_loglog(logx, logy, prefactor, exponent, **fit_opts)
+    prefactor, exponent, details["prefactor_error"], details["exponent_error"] = _fit_loglog(
+        logx, logy, prefactor, exponent, **fit_opts
+    )
+
+    details["prefactor"] = prefactor
+    details["exponent"] = exponent
+
+    if auto_fmt:
+        assert fmt is None
+        fmt = "".join(
+            [
+                r"$({prefactor:.2f} \pm {prefactor_error:.2f})",
+                auto_fmt,
+                r"^{{{exponent:.2f} \pm {prefactor_error:.2f}}}$",
+            ]
+        )
+
+    if fmt:
+        assert "label" not in kwargs
+        label = fmt.format(**details)
+        kwargs["label"] = label
+        details["label"] = label
 
     if axis is None:
-        return (prefactor, exponent)
+        return (prefactor, exponent, details)
 
     xp = np.array([np.min(np.exp(logx)), np.max(np.exp(logx))])
     xl = np.array([axis.get_xlim()[0], xp[0]])
     xu = np.array([xp[1], axis.get_xlim()[1]])
-    details = {}
 
     if isinstance(extrapolate, bool) and extrapolate:
         xp = np.array(axis.get_xlim())
@@ -1340,16 +1396,6 @@ def fit_powerlaw(
     yl = prefactor * xl ** exponent
     yu = prefactor * xu ** exponent
 
-    if auto_fmt:
-        assert fmt is None
-        fmt = r"${0:.3f} " + auto_fmt + r"^{{{1:.2f}}}$"
-
-    if fmt:
-        assert "label" not in kwargs
-        label = fmt.format(prefactor, exponent)
-        kwargs["label"] = label
-        details["label"] = label
-
     details["handle"] = axis.plot(xp, yp, **kwargs)
 
     if isinstance(extrapolate, dict):
@@ -1363,6 +1409,7 @@ def fit_exp(
     xdata: ArrayLike,
     ydata: ArrayLike,
     yerr: ArrayLike = None,
+    yerr_mode: str = "differentials",
     prefactor: float = None,
     exponent: float = None,
     axis: plt.Axes = None,
@@ -1370,20 +1417,32 @@ def fit_exp(
     auto_fmt: str = None,
     extrapolate: bool | dict = False,
     **kwargs,
-):
+) -> (float, float, dict):
     r"""
-    Fit an exponential :math:`y = c \exp(b x)` by converting
-    :math:`y` to its logarithm and fitting a straight line.
+    Fit an exponential :math:`y = c \exp(b x)` by linear fitting of
+    :math`ln y = ln c + b x`.
     This function does not support more customised operation like fitting an offset,
     but custom code can be easily written by copy/pasting from here.
 
-    Warning: If this function is used to plot the fit, beware that the fit is plotted using just
-    two data-points if the axis is set to semilogy-scale
-    (as the fit will be a straight line on that scale).
+    .. warning::
+        If this function is used to plot the fit, beware that the fit is plotted using just
+        two data-points if the axis is set to semilogy-scale
+        (as the fit will be a straight line on that scale).
+
+    Different modes are available to treat ``yerr``:
+
+    *   ``"differentials"``: assume that ``yerr << ydata``, such that
+
+        .. math::
+
+            z &\equiv \ln y \\
+            \delta z &= \left| \frac{\partial z}{\partial y} \right| \delta y \\
+            \delta z &= \frac{\delta y}{y}
 
     :param xdata: Data points along the x-axis.
     :param ydata: Data points along the y-axis.
     :param yerr: Error-bar for ``ydata``.
+    :param yerr_mode: How to treat the error in ``ydata``, see above.
     :param prefactor: Prefactor :math:`c` (fitted if not specified).
     :param exponent: Exponent :math:`b` (fitted if not specified).
     :param axis: Axis to plot along (not plotted if not specified).
@@ -1401,12 +1460,19 @@ def fit_exp(
         Other plot options.
 
     :return:
-        ``prefactor, exponent[, plot_details]``
+        ``prefactor, exponent, details``
         The (fitted) prefector and exponent.
-        If plotting, the following details are return as dictionary::
+        The details are a dictionary as follows::
 
-            handle: Handle of the plot.
-            label: Label (if ``fmt`` was specified).
+            prefactor: (Fitted) prefactor.
+            exponent: (Fitted) exponent.
+            prefactor_error: Estimated error of prefactor.
+            exponent_error: Estimated error of exponent.
+            pcov: Covariance of fit.
+            label: Label.
+            handle: Handle of the plot (if ``axis`` was specified).
+            handle_lower: Handle of the plot of the lower extrapolation, if present.
+            handle_upper: Handle of the plot of the upper extrapolation, if present.
     """
 
     xdata = np.array(xdata)
@@ -1416,27 +1482,53 @@ def fit_exp(
     x = xdata[i]
     logy = np.log(ydata[i])
 
-    j = np.isnan(logy)
+    j = np.logical_or(np.isnan(x), np.isnan(logy))
     logy = logy[~j]
     x = x[~j]
 
     fit_opts = {}
+    details = {}
 
     if yerr is not None:
-        logyerr = np.log(np.array(yerr)[i])
-        logyerr = logyerr[~j]
-        fit_opts["sigma"] = logyerr
-        fit_opts["absolute_sigma"] = True
+        if yerr_mode.lower() == "differentials":
+            yerr = np.array(yerr)
+            sigma = yerr[i][~j] / ydata[i][~j]
+            sigma[yerr[i][~j] == 0] = np.finfo(sigma.dtype).eps  # avoid zero division
+            fit_opts["sigma"] = sigma
+            fit_opts["absolute_sigma"] = False
+        else:
+            raise OSError("yerr_mode: did you mean 'differentials'?")
 
-    prefactor, exponent = _fit_loglog(x, logy, prefactor, exponent, **fit_opts)
+    prefactor, exponent, details["prefactor_error"], details["exponent_error"] = _fit_loglog(
+        x, logy, prefactor, exponent, **fit_opts
+    )
+
+    details["prefactor"] = prefactor
+    details["exponent"] = exponent
+
+    if auto_fmt:
+        assert fmt is None
+        fmt = "".join(
+            [
+                r"$({prefactor:.2f} \pm {prefactor_error:.2f})",
+                r"\exp( {exponent:.2f} \pm {prefactor_error:.2f}",
+                auto_fmt,
+                ")$",
+            ]
+        )
+
+    if fmt:
+        assert "label" not in kwargs
+        label = fmt.format(**details)
+        kwargs["label"] = label
+        details["label"] = label
 
     if axis is None:
-        return (prefactor, exponent)
+        return (prefactor, exponent, details)
 
     xp = np.array([np.min(x), np.max(x)])
     xl = np.array([axis.get_xlim()[0], xp[0]])
     xu = np.array([xp[1], axis.get_xlim()[1]])
-    details = {}
 
     if isinstance(extrapolate, bool) and extrapolate:
         xp = np.array(axis.get_xlim())
@@ -1450,16 +1542,6 @@ def fit_exp(
     yl = prefactor * np.exp(exponent * xl)
     yu = prefactor * np.exp(exponent * xu)
 
-    if auto_fmt:
-        assert fmt is None
-        fmt = r"${0:.3f} \exp ({1:.2f} " + auto_fmt + r")$"
-
-    if fmt:
-        assert "label" not in kwargs
-        label = fmt.format(prefactor, exponent)
-        kwargs["label"] = label
-        details["label"] = label
-
     details["handle"] = axis.plot(xp, yp, **kwargs)
 
     if isinstance(extrapolate, dict):
@@ -1480,7 +1562,7 @@ def fit_linear(
     auto_fmt: str = None,
     extrapolate: bool | dict = False,
     **kwargs,
-):
+) -> (float, float, dict):
     r"""
     Fit a linear function :math:`y = a + b x`.
 
@@ -1490,69 +1572,104 @@ def fit_linear(
     :param offset: Offset :math:`a` (fitted if not specified).
     :param slope: Slope :math:`b` (fitted if not specified).
     :param axis: Axis to plot along (not plotted if not specified).
-    :param fmt: Format for the label (if plotting). E.g. ``r"${0:.3f} + {1:.2f} x$"``.
+    :param fmt: Format for the label, e.g. ``r"${offset:.2f} + {slope:.2f} x$"``.
     :param auto_fmt:
-        Format label (if plotting) as ``r"${0:.3f} + {1:.2f} x$"``,
+        Format label as ``r"$({offset:.2f} \pm {offset_error:.2f}) +
+        ({slope:.2f} \pm {slope_error:.2f}) x$"``,
         with ``x`` replaced with the specified string.
 
     :param extrapolate:
         Plot the powerlaw on the full range of ``axis.get_xlim()``.
-        Instead of ``True'', one can specify plot options for the extrapolated line, e.g.
+        Instead of ``True``, one can specify plot options for the extrapolated line, e.g.
         ``..., extrapolate=dict(ls="--", c="r"), ...``.
 
     :param kwargs:
         Other plot options.
 
     :return:
-        ``offset, slope[, plot_details]``
+        ``offset, slope, details``
         The (fitted) offset and slope.
-        If plotting, the following details are return as dictionary::
+        The details are a dictionary as follows::
 
-            handle: Handle of the plot.
-            label: Label (if ``fmt`` was specified).
+            offset: (Fitted) offset.
+            slope: (Fitted) slope.
+            offset_error: Estimated error of offset.
+            slope_error: Estimated error of slope.
+            pcov: Covariance of fit.
+            label: Label.
+            handle: Handle of the plot (if ``axis`` was specified).
+            handle_lower: Handle of the plot of the lower extrapolation, if present.
+            handle_upper: Handle of the plot of the upper extrapolation, if present.
     """
 
     xdata = np.array(xdata)
     ydata = np.array(ydata)
 
     fit_opts = {}
+    details = {}
 
     if yerr is not None:
-        fit_opts["sigma"] = np.array(yerr)
-        fit_opts["absolute_sigma"] = True
+        sigma = np.array(yerr).astype(float)
+        sigma[yerr == 0] = np.finfo(sigma.dtype).eps  # avoid zero division
+        fit_opts["sigma"] = sigma
+        fit_opts["absolute_sigma"] = False
 
     if offset is None and slope is None:
 
         def f(x, offset, slope):
             return offset + slope * x
 
-        param, _ = curve_fit(f, xdata, ydata, **fit_opts)
+        param, pcov = curve_fit(f, xdata, ydata, **fit_opts)
         offset = param[0]
         slope = param[1]
+        details["offset_error"] = np.sqrt(pcov[0, 0])
+        details["slope_error"] = np.sqrt(pcov[1, 1])
 
     elif offset is None:
 
         def f(x, offset):
             return offset + slope * x
 
-        param, _ = curve_fit(f, xdata, ydata, **fit_opts)
+        param, pcov = curve_fit(f, xdata, ydata, **fit_opts)
         offset = param[0]
+        details["offset_error"] = np.sqrt(pcov[0, 0])
+        details["slope_error"] = 0
 
     elif slope is None:
 
         def f(x, slope):
             return offset + slope * x
 
-        param, _ = curve_fit(f, xdata, ydata, **fit_opts)
+        param, pcov = curve_fit(f, xdata, ydata, **fit_opts)
         slope = param[0]
+        details["offset_error"] = 0
+        details["slope_error"] = np.sqrt(pcov[0, 0])
+
+    details["offset"] = offset
+    details["slope"] = slope
+
+    if auto_fmt:
+        assert fmt is None
+        fmt = "".join(
+            [
+                r"$({offset:.2f} \pm {offset_error:.2f}) + ({slope:.2f} + {slope_error:.2f}) ",
+                auto_fmt,
+                r"$",
+            ]
+        )
+
+    if fmt:
+        assert "label" not in kwargs
+        label = fmt.format(**details)
+        kwargs["label"] = label
+        details["label"] = label
 
     if axis is None:
-        return (offset, slope)
+        return (offset, slope, details)
 
     xp = np.array([np.min(xdata), np.max(xdata)])
     xl = np.array([axis.get_xlim()[0], xp[0]])
     xu = np.array([xp[1], axis.get_xlim()[1]])
-    details = {}
 
     if isinstance(extrapolate, bool) and extrapolate:
         xp = np.array(axis.get_xlim())
@@ -1561,16 +1678,6 @@ def fit_linear(
     yl = offset + slope * xl
     yu = offset + slope * xu
 
-    if auto_fmt:
-        assert fmt is None
-        fmt = r"${0:.3f} + {1:.2f} " + auto_fmt + r"$"
-
-    if fmt:
-        assert "label" not in kwargs
-        label = fmt.format(offset, slope)
-        kwargs["label"] = label
-        details["label"] = label
-
     details["handle"] = axis.plot(xp, yp, **kwargs)
 
     if isinstance(extrapolate, dict):
@@ -1969,25 +2076,90 @@ def hist(P, edges, **kwargs):
     return p
 
 
-def cdf(data, mode="continuous", **kwargs):
+def cdf(data: ArrayLike, less_equal: bool = False) -> (np.ndarray, np.ndarray):
     """
-    Return cumulative density.
+    Cumulative distribution function: ``P(x < X)``.
 
-    :arguments:
-
-        **data** (``<numpy.ndarray>``)
-            Input data, to plot the distribution for.
-
-    :returns:
-
-        **P** (``<numpy.ndarray>``)
-            Cumulative probability.
-
-        **x** (``<numpy.ndarray>``)
-            Data points.
+    :param data: Input data.
+    :param less_equal: If ``True`` return ``P(x <= X)``, if ``False return ``P(x < X)``.
+    :return: ``(P, X)``
     """
 
-    return (np.linspace(0.0, 1.0, len(data)), np.sort(data))
+    bin_edges = np.unique(data)
+    ibin = np.digitize(data, bin_edges, right=not less_equal)
+    _, count = np.unique(ibin, return_counts=True)
+
+    if less_equal:
+        return np.cumsum(count) / data.size, bin_edges
+    else:
+        return np.cumsum([0] + count.tolist())[:-1] / data.size, bin_edges
+
+
+def ccdf(data: ArrayLike, greater_equal: bool = True) -> (np.ndarray, np.ndarray):
+    """
+    Complementary cumulative distribution function: ``P(x >= X)``.
+    By definition: ``ccdf(data)[0] == 1 - cdf(data)[0])``.
+
+    :param data: Input data.
+    :param greater_equal: If ``True`` return ``P(x >= X)``, if ``False return ``P(x > X)``.
+    :return: ``(P, X)``
+    """
+
+    bin_edges = np.unique(data)
+    ibin = np.digitize(data, bin_edges[::-1], right=greater_equal)
+    _, count = np.unique(ibin, return_counts=True)
+
+    if greater_equal:
+        return np.cumsum(count)[::-1] / data.size, bin_edges
+    else:
+        return np.cumsum([0] + count.tolist())[1::-1] / data.size, bin_edges
+
+
+def bin(x: ArrayLike, y: ArrayLike, bin_edges: ArrayLike | int, use_median: bool = False):
+    """
+    Bin data.
+
+    :param x: x-data.
+    :param y: y-data.
+    :param bin_edges: Bin-edges along the x-axis, or the number of bins.
+    :param use_median: Use median instead of mean.
+    :return: Dictionary as follows::
+        x: mean(x) for each bin (or median(x) if use_median = True).
+        y: mean(y) for each bin (or median(y) if use_median = True).
+        xerr: std(x) for each bin.
+        yerr: std(y) for each bin.
+    """
+
+    j = np.digitize(x, bin_edges) - 1
+    n = bin_edges.size - 1
+
+    ret = {
+        "x": np.NaN * np.ones(n, dtype=float),
+        "y": np.NaN * np.ones(n, dtype=float),
+        "xerr": np.NaN * np.ones(n, dtype=float),
+        "yerr": np.NaN * np.ones(n, dtype=float),
+    }
+
+    assert np.max(j) < ret["x"].size
+
+    for i in range(np.max(j) + 1):
+
+        if i not in j:
+            continue
+
+        sel = j == i
+
+        if not use_median:
+            ret["x"][i] = np.mean(x[sel])
+            ret["y"][i] = np.mean(y[sel])
+        else:
+            ret["x"][i] = np.median(x[sel])
+            ret["y"][i] = np.median(y[sel])
+
+        ret["xerr"][i] = np.std(x[sel])
+        ret["yerr"][i] = np.std(y[sel])
+
+    return ret
 
 
 def patch(*args, **kwargs):
